/**
 * Class with auxiliary SOAP functions.
 */

package ru.utility;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SignatureException;
import java.security.cert.CRL;
import java.security.cert.CRLException;
import java.security.cert.CertPath;
import java.security.cert.CertPathBuilderException;
import java.security.cert.CertPathValidator;
import java.security.cert.CertPathValidatorException;
import java.security.cert.CertStore;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.CollectionCertStoreParameters;
import java.security.cert.PKIXCertPathValidatorResult;
import java.security.cert.PKIXParameters;
import java.security.cert.TrustAnchor;
import java.security.cert.X509CertSelector;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.apache.axis.Message;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.ws.security.WSSecurityException;
import org.apache.ws.security.components.crypto.Merlin;

public class SOAPUtility {

	/**
	 * Simple XML SOAP example.
	 */
	public static final String SOAPMSG = 
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" 
        + "<SOAP-ENV:Envelope "
        +   "xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" "
        +   "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" "
        +   "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">" 
        +   "<SOAP-ENV:Body>" 
        +       "<add xmlns=\"http://ws.apache.org/counter/counter_port_type\">" 
        +           "<value xmlns=\"\">15</value>" 
        +       "</add>" 
        +   "</SOAP-ENV:Body>" 
        + "</SOAP-ENV:Envelope>";
	
	/** Function checks 'org.apache.ws.security.crypto.provider' option in 
	 * 'crypto.properties' to determine wss4j version: 
	 * 		if Merlin's base class is AbstractCrypto, then wss4j version is 1.5.11, 
	 * 		else if Merlin's base class is CryptoBase, then version is 1.6.3.
	 * @param className - name of provider (Merlin).
	 * @return - true, if version is 1.5.11.
	 * @throws ClassNotFoundException
	 */
	public static boolean is_1_5_11( String className ) throws ClassNotFoundException {
	
		Class classType = Class.forName(className);
		Class superClassType = classType.getSuperclass();
		String superClassName = superClassType.getSimpleName();
		
		// If properties contains MerlinEx class
		if ( superClassName.equalsIgnoreCase("Merlin") ) {
			return is_1_5_11(superClassType.getName());
		}
		
		if ( superClassName.equalsIgnoreCase("AbstractCrypto") ) {
			return true;
		}
		
		return false;
	}
	
	/**
	 * Function creates SOAP XML message from string.
	 * @return SOAP XML message.
	 * @throws Exception
	 */
	public static SOAPEnvelope getSOAPEnvelopeFromString(String message) throws Exception {
        
    	InputStream input = new ByteArrayInputStream(message.getBytes());
        Message msg = new Message(input);
        return msg.getSOAPEnvelope();
        
    }
	
	/**
	 * Function creates SOAP XML message reading file .
	 * @param fileName - SOAP XML file name.
	 * @return SOAP XML message.
	 * @throws Exception
	 */
	public static SOAPEnvelope getSOAPEnvelopeFromFile(String fileName) throws Exception {
    	
    	InputStream input = new FileInputStream( new File(fileName) );
        Message msg = new Message(input);
        return msg.getSOAPEnvelope();
        
    }
	
	/**
	 * Function verifies the user certificate chain 'certs'.
	 * @param crlFile - CRL file (can be null).
	 * @param certs - list of user certificates to be checked.
	 * @param cacerts - list of trusted certificates. 
	 * @param endPointCertificate - a certificate as last verifiable link in the chain (can be null).
	 * @return true, if chain is valid.
	 * @throws WSSecurityException
	 * @throws CRLException
	 * @throws FileNotFoundException
	 * @throws SignatureException 
	 * @throws NoSuchProviderException 
	 * @throws InvalidKeyException 
	 * @throws CertPathBuilderException 
	 */
	public static boolean validateCertPath(File crlFile,
			X509Certificate[] certs, X509Certificate[] cacerts,
			X509Certificate endPointCertificate) throws 
			WSSecurityException, CRLException, FileNotFoundException, InvalidKeyException, 
			NoSuchProviderException, SignatureException, CertPathBuilderException {
        
		PKIXCertPathValidatorResult bldResult = null;
		List<CertStore> storeCertCRL = null;
		CertStore crlCertStore = null;
    	boolean crlIsAdded = false;
    	boolean bResult = false;
		
		try {
          
    		CertificateFactory certFactory = CertificateFactory.getInstance("X509");
    		
    		/**
    		 * Add CRL in list.
    		 */
    		if ( crlFile != null ) {
    			
    			CRL crl = certFactory.generateCRL(new FileInputStream(crlFile));
    			if ( crl != null )
    				crlCertStore = CertStore.getInstance( "Collection", new CollectionCertStoreParameters(Collections.singletonList(crl)) );
    		
    			if (crlCertStore != null) {
    				storeCertCRL = Arrays.asList(crlCertStore);
    				crlIsAdded = true;
    			}
    		}
    		
            List<X509Certificate> certList = Arrays.asList(certs);
            CertPath path = certFactory.generateCertPath(certList);
            System.out.println("Certificate path length: " + path.getCertificates().size());
            
            Set<TrustAnchor> trustedAnchors = new HashSet<TrustAnchor>();

            if (cacerts != null) {
                for ( int i = 0; i < cacerts.length; ++i ) {
                    TrustAnchor anchor = 
                        new TrustAnchor(cacerts[i], 
                        		cacerts[i].getExtensionValue(Merlin.NAME_CONSTRAINTS_OID));
                    if (anchor != null && !trustedAnchors.contains(anchor) )
                    	trustedAnchors.add(anchor);
                }
            }

            /* 
            if (certs != null) {
                for ( int i = 0; i < certs.length; ++i ) {
                	TrustAnchor anchor = 
                		new TrustAnchor(certs[i], 
                    		certs[i].getExtensionValue(Merlin.NAME_CONSTRAINTS_OID));
                	if ( anchor != null && !trustedAnchors.contains(anchor) )
                		trustedAnchors.add(anchor);
                }
            }
            */
            
            System.out.println("Trusted anchor deepth: " + trustedAnchors.size());
            
            if (certs != null && cacerts != null && 
            		certs.length == 1 && cacerts.length == 1) {
            	certs[0].checkValidity();
            	certs[0].verify(cacerts[0].getPublicKey());
            }
            
            PKIXParameters param = new PKIXParameters(trustedAnchors);
            param.setSigProvider(null);
            param.setRevocationEnabled(crlIsAdded);
            if (crlIsAdded)
            	 param.setCertStores(storeCertCRL);
            
            if (endPointCertificate != null) {
            	final X509CertSelector selector = new X509CertSelector();
            	selector.setCertificate(endPointCertificate);
            	param.setTargetCertConstraints(selector);
            }

            CertPathValidator certPathValidator = CertPathValidator.getInstance("PKIX");
            bldResult = (PKIXCertPathValidatorResult)certPathValidator.validate(path, param);
            /* MUST BE (bldResult != null)
            PublicKey publicKey = bldResult.getPublicKey();
            */
            bResult = true;
            
        } catch (NoSuchAlgorithmException ex) {
            ex.printStackTrace();
        } catch (CertificateException ex) {
        	ex.printStackTrace();
        } catch (InvalidAlgorithmParameterException ex) {
        	ex.printStackTrace();
        } catch (CertPathValidatorException ex) {
        	ex.printStackTrace();
        	System.out.println("Index of certificate that caused exception: "
                    + ex.getIndex());
        }
        
        System.out.println("Path Validator result (public key algorithm): " + 
        		((bldResult != null) ? bldResult.getPublicKey().getAlgorithm() : null));

        return bResult;
    }
}
